@@ -66,7 +66,6 @@ typedef struct {
     unsigned int keyval;
     unsigned int sequence;
     unsigned int time;
-    PyObject*    touch;
 
     PyObject*    source_device;
 
@@ -80,7 +79,6 @@ osk_device_event_init (OskDeviceEvent* self, PyObject *args, PyObject *kwds)
     self->xid_event = None;
     self->device_id = 0;
     self->source_id = 0;
-    self->touch = Py_None;
     self->source_device = Py_None;
     Py_INCREF(self->source_device);
     return 0;
@@ -105,6 +103,35 @@ new_device_event (void)
     return NULL;
 }
 
+static PyObject *
+osk_device_event_copy (OskDeviceEvent* self, PyObject *args)
+{
+    OskDeviceEvent *ev = new_device_event();
+    if (ev)
+    {
+        ev->display = self->display;
+        ev->xid_event = self->xid_event;
+        ev->xi_type = self->xi_type;
+        ev->type = self->type;
+        ev->device_id = self->device_id;
+        ev->source_id = self->source_id;
+        ev->x = self->x;
+        ev->y = self->y;
+        ev->x_root = self->x_root;
+        ev->y_root = self->y_root;
+        ev->button = self->button;
+        ev->state = self->state;
+        ev->keyval = self->keyval;
+        ev->sequence = self->sequence;
+        ev->time = self->time;
+
+        Py_DECREF(ev->source_device);
+        ev->source_device = self->source_device;
+        Py_INCREF(ev->source_device);
+    }
+    return (PyObject*) ev;
+}
+
 static PyObject *
 osk_device_event_get_time (OskDeviceEvent* self, PyObject *args)
 {
@@ -128,6 +155,8 @@ osk_device_event_get_source_device (OskDeviceEvent* self, PyObject *args)
 }
 
 static PyMethodDef osk_device_event_methods[] = {
+    { "__copy__",
+      (PyCFunction) osk_device_event_copy, METH_NOARGS,  NULL },
     { "get_time",
       (PyCFunction) osk_device_event_get_time, METH_NOARGS,  NULL },
     { "get_source_device",
@@ -139,20 +168,19 @@ static PyMethodDef osk_device_event_methods[] = {
 
 static PyMemberDef osk_device_event_members[] = {
     {"xid_event", T_UINT, offsetof(OskDeviceEvent, xid_event), READONLY, NULL },
-    {"xi_type", T_UINT, offsetof(OskDeviceEvent, xi_type), READONLY, NULL },
-    {"type", T_UINT, offsetof(OskDeviceEvent, type), READONLY, NULL },
+    {"xi_type", T_UINT, offsetof(OskDeviceEvent, xi_type), RESTRICTED, NULL },
+    {"type", T_UINT, offsetof(OskDeviceEvent, type), RESTRICTED, NULL },
     {"device_id", T_UINT, offsetof(OskDeviceEvent, device_id), READONLY, NULL },
     {"source_id", T_UINT, offsetof(OskDeviceEvent, source_id), READONLY, NULL },
     {"x", T_DOUBLE, offsetof(OskDeviceEvent, x), RESTRICTED, NULL },
     {"y", T_DOUBLE, offsetof(OskDeviceEvent, y), RESTRICTED, NULL },
     {"x_root", T_DOUBLE, offsetof(OskDeviceEvent, x_root), RESTRICTED, NULL },
     {"y_root", T_DOUBLE, offsetof(OskDeviceEvent, y_root), RESTRICTED, NULL },
-    {"button", T_UINT, offsetof(OskDeviceEvent, button), READONLY, NULL },
+    {"button", T_UINT, offsetof(OskDeviceEvent, button), RESTRICTED, NULL },
     {"state", T_UINT, offsetof(OskDeviceEvent, state), RESTRICTED, NULL },
     {"keyval", T_UINT, offsetof(OskDeviceEvent, keyval), READONLY, NULL },
-    {"sequence", T_UINT, offsetof(OskDeviceEvent, sequence), READONLY, NULL },
+    {"sequence", T_UINT, offsetof(OskDeviceEvent, sequence), RESTRICTED, NULL },
     {"time", T_UINT, offsetof(OskDeviceEvent, time), READONLY, NULL },
-    {"touch", T_OBJECT, offsetof(OskDeviceEvent, touch), READONLY, NULL },
     {NULL}
 };
 
@@ -205,6 +233,7 @@ osk_devices_init (OskDevices *dev, PyObject *args, PyObject *kwds)
     int event, error;
     int major = 2;
     int minor = 2;
+    Status status;
 
     /* set display before anything else! */
     GdkDisplay* display = gdk_display_get_default ();
@@ -228,7 +257,7 @@ osk_devices_init (OskDevices *dev, PyObject *args, PyObject *kwds)
     // successfully calling it before us, so just ignore the
     // error and move on.
     gdk_error_trap_push ();
-    Status status = XIQueryVersion (dev->dpy, &major, &minor);
+    status = XIQueryVersion (dev->dpy, &major, &minor);
     gdk_error_trap_pop_ignored ();
     if (status == BadRequest)
     {
@@ -340,11 +369,12 @@ static gboolean idle_process_event_queue (OskDevices* dev)
 
     for (;;)
     {
+        PyObject* arglist;
         OskDeviceEvent* event = g_queue_pop_tail (queue);
         if (!event)
             break;
 
-        PyObject* arglist = Py_BuildValue("(O)", event);
+        arglist = Py_BuildValue("(O)", event);
         if (arglist)
         {
             Py_INCREF(dev->event_handler);
@@ -432,10 +462,6 @@ osk_devices_call_event_handler_pointer (OskDevices  *dev,
         ev->sequence = sequence;
         ev->time = time;
 
-        // Link event to itself in the touch property for
-        // compatibility with GDK touch events.
-        ev->touch = (PyObject*) ev;
-
         queue_event (dev, ev, type == XI_Motion);
 
         Py_DECREF(ev);
@@ -447,6 +473,7 @@ osk_devices_call_event_handler_key (OskDevices *dev,
                                     int         type,
                                     Display*    display,
                                     int         device_id,
+                                    int         source_id,
                                     int         keyval
 )
 {
@@ -457,6 +484,7 @@ osk_devices_call_event_handler_key (OskDevices *dev,
         ev->xi_type = type;
         ev->type = translate_event_type(type);
         ev->device_id = device_id;
+        ev->source_id = source_id;
         ev->keyval = keyval;
 
         queue_event (dev, ev, False);
@@ -573,11 +601,13 @@ osk_devices_translate_keycode (int              keycode,
  * Get Gdk event state of the master pointer.
  *
  * The master aggregates currently pressed buttons and key presses from all
- * slave devices, something we would have to do ourselves otherwise.
+ * slave devices.
  *
- * Reason: Francesco uses one pointing device for button presses and another
+ * Why we need aggregate button presses:
+ * Francesco uses one pointing device for button presses and a different one
  * for motion events. The motion slave doesn't know about the button
- * slave's state, requiring us to get the aggregate state of all slaves.
+ * slave's state. We can either aggregate the button state over all slaves
+ * ourselves or rely on the master to do it for us.
  */
 static unsigned int
 get_master_state (OskDevices* dev)
@@ -684,20 +714,21 @@ handle_pointing_event (int evtype, XIEvent* xievent, OskDevices* dev)
         case XI_TouchEnd:
         {
             XIDeviceEvent *event = (XIDeviceEvent*) xievent;
-
             unsigned int button = 0;
+            unsigned int sequence = 0;
+            unsigned int state;
+
             if (evtype == XI_ButtonPress ||
                 evtype == XI_ButtonRelease)
                 button = event->detail;
 
-            unsigned int sequence = 0;
             if (evtype == XI_TouchBegin ||
                 evtype == XI_TouchUpdate ||
                 evtype == XI_TouchEnd)
                 sequence = event->detail;
 
             update_state(evtype, event, dev);
-            unsigned int state = get_current_state (dev);
+            state = get_current_state (dev);
 
             osk_devices_call_event_handler_pointer (dev,
                                                     evtype,
@@ -862,6 +893,7 @@ osk_devices_event_filter (GdkXEvent  *gdk_xevent,
                                                             evtype,
                                                             event->display,
                                                             event->deviceid,
+                                                            event->sourceid,
                                                             keyval);
                 }
                 break;
@@ -880,6 +912,7 @@ osk_devices_event_filter (GdkXEvent  *gdk_xevent,
                                                         evtype,
                                                         event->display,
                                                         event->deviceid,
+                                                        event->sourceid,
                                                         keyval);
                 break;
             }
@@ -1145,25 +1178,30 @@ osk_devices_grab_device (PyObject *self, PyObject *args)
     OskDevices       *dev = (OskDevices *) self;
     int               id;
     int               _win;
+    Window win;
+
+    unsigned char mask[1] = {0};
+    XIEventMask events;
+
+    Status status;
+    gint error;
 
     if (!PyArg_ParseTuple (args, "ii", &id, &_win))
         return NULL;
 
-    Window win = (Window)_win;
+    win = (Window)_win;
     if (!win)
         win = DefaultRootWindow (dev->dpy);
 
-    unsigned char mask[1] = {0};
-    XIEventMask events;
     events.deviceid = id;
     events.mask = mask;
     events.mask_len = sizeof(mask);
 
     gdk_error_trap_push ();
-    Status status = XIGrabDevice(dev->dpy, id, win, CurrentTime, None,
+    status = XIGrabDevice(dev->dpy, id, win, CurrentTime, None,
                                  XIGrabModeSync, XIGrabModeAsync,
                                  True, &events);
-    gint error = gdk_error_trap_pop ();
+    error = gdk_error_trap_pop ();
 
     if (status != Success || error)
     {
@@ -1184,15 +1222,17 @@ osk_devices_grab_device (PyObject *self, PyObject *args)
 static PyObject *
 osk_devices_ungrab_device (PyObject *self, PyObject *args)
 {
-    OskDevices       *dev = (OskDevices *) self;
-    int               id;
+    OskDevices *dev = (OskDevices *) self;
+    int         id;
+    Status      status;
+    gint        error;
 
     if (!PyArg_ParseTuple (args, "i", &id))
         return NULL;
 
     gdk_error_trap_push ();
-    Status status = XIUngrabDevice(dev->dpy, id, CurrentTime);
-    gint error = gdk_error_trap_pop ();
+    status = XIUngrabDevice(dev->dpy, id, CurrentTime);
+    error = gdk_error_trap_pop ();
     if (status != Success || error)
     {
         PyErr_Format (OSK_EXCEPTION, "failed to ungrab device (0x%x, 0x%x)",
@@ -1219,11 +1259,12 @@ osk_devices_select_events (PyObject *self, PyObject *args)
     int           device_id;
     unsigned long event_mask;
     int           _win;
+    Window        win;
 
     if (!PyArg_ParseTuple (args, "iil", &_win, &device_id, &event_mask))
         return NULL;
 
-    Window win = (Window) _win;
+    win = (Window) _win;
 
     if (dev->event_handler)
     {
@@ -1260,11 +1301,12 @@ osk_devices_unselect_events (PyObject *self, PyObject *args)
     unsigned char mask[1] = { 0 };
     int           device_id;
     int           _win;
+    Window        win;
 
     if (!PyArg_ParseTuple (args, "ii", &_win, &device_id))
         return NULL;
 
-    Window win = (Window) _win;
+    win = (Window) _win;
 
     if (dev->event_handler)
     {
