@@ -912,9 +912,9 @@ UnigramModel_count_ngram(PyUnigramModel* self, PyObject* args)
 {
     PyObject* ngram = NULL;
     int increment = 1;
-    bool allow_new_words = true;
+    int allow_new_words = true;
 
-    if (! PyArg_ParseTuple(args, "O|IB:count_ngram",
+    if (! PyArg_ParseTuple(args, "O|II:count_ngram",
               &ngram, &increment, &allow_new_words))
         return NULL;
 
@@ -1119,9 +1119,9 @@ DynamicModel_count_ngram(PyDynamicModel* self, PyObject* args)
 {
     PyObject* ngram = NULL;
     int increment = 1;
-    bool allow_new_words = true;
+    int allow_new_words = true;
 
-    if (! PyArg_ParseTuple(args, "O|IB:count_ngram",
+    if (! PyArg_ParseTuple(args, "O|II:count_ngram",
               &ngram, &increment, &allow_new_words))
         return NULL;
 
@@ -2026,6 +2026,8 @@ moduleinit (void)
                              PyInt_FromLong(LanguageModel::NORMALIZE));
         PyDict_SetItemString(LanguageModelType.tp_dict, "NO_SORT",
                              PyInt_FromLong(LanguageModel::NO_SORT));
+        PyDict_SetItemString(LanguageModelType.tp_dict, "NUM_CONTROL_WORDS",
+                             PyInt_FromLong(NUM_CONTROL_WORDS));
     }
 
     return module;
